﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Infragistics.Windows.DataPresenter;

namespace WpfApplication2
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            SummaryCalculator.Register(new CustomSummaryCalculator());
            this.DataContext = ChartData.GenerateData(2);
        }
    }

    public class ChartData
    {
        public DateTime Date { get; set; }
        public double Value1 { get; set; }
        public double Value2 { get; set; }
        public double Value3 { get; set; }

        public static ObservableCollection<ChartData> GenerateData(int count)
        {
            ObservableCollection<ChartData> data = new ObservableCollection<ChartData>();

            Random r = new Random();

            ChartData data0 = new ChartData
            {
                Date = DateTime.Now,
                Value1 = r.Next(20, 40),
                Value2 = r.Next(20, 40),
                Value3 = r.Next(20, 40),
            };

            data.Add(data0);

            for (int i = 1; i < count; i++)
            {
                ChartData dat = new ChartData
                {
                    Date = DateTime.Now.AddDays(i),
                };

                if (r.NextDouble() > .5)
                {
                    dat.Value1 = data[i - 1].Value1 + r.NextDouble();
                    dat.Value2 = data[i - 1].Value2 + r.NextDouble();
                    dat.Value3 = data[i - 1].Value3 + r.NextDouble();
                }
                else
                {
                    dat.Value1 = data[i - 1].Value1 - r.NextDouble();
                    dat.Value2 = data[i - 1].Value2 - r.NextDouble();
                    dat.Value3 = data[i - 1].Value3 - r.NextDouble();
                }

                data.Add(dat);
            }

            return data;
        }
    }

    public class CustomSummaryCalculator : SummaryCalculator
    {

        public CustomSummaryCalculator() : base() { }

        double column1, column2;

        public override void BeginCalculation(SummaryResult summaryResult)
        {

            column1 = 0;

            column2 = 0;

        }

        public override bool CanProcessDataType(Type dataType)
        {

            return dataType == typeof(double);

        }

        public override void Aggregate(object dataValue, Infragistics.Windows.DataPresenter.SummaryResult summaryResult, Record record)
        {

            if (dataValue != null && (record as DataRecord) != null)
            {

                var dataItem = (record as DataRecord).DataItem as ChartData;

                if (dataItem != null)
                {

                    column2 += dataItem.Value2; 
                    column1 += dataItem.Value1;

                }

            }

        }

        public override object EndCalculation(Infragistics.Windows.DataPresenter.SummaryResult summaryResult)
        {

            return column2 != 0 ? column1 / column2 : 0;

        }

        public override string Name { get { return "CustomSum"; } }

        public override string Description { get { return "Calculation of Rate"; } }

    }

}
